/**
* Example script demonstrating various ways of communicating with a PIXILAB Blocks
* from a custom web page. This custom web page (assuming it consists of only static
* files) can be server by Blocks simply by putting all the files in a directory
* under /public; here we use /public/pubsubpeer-test.
*
* This script should run on any modern, ES6-compatible browser.
*/

// First we wait for the dom content to load.
document.addEventListener("DOMContentLoaded", function () {

	//////////////////////////////////////////////////////////
	// GENERAL SETUP
	//////////////////////////////////////////////////////////

	/**
	 * Instantiate a PubSubPeer that auto-connects to the blocks server
	 * using a websocket.
	 *
	 * The PIXILAB Websocket API is defined in the pub-sub-peer.js script, which
	 * attaches its root object as a global (window) variable named PIXILAB_BLOCKS.
	 */
	const pubSubPeer = new PIXILAB_BLOCKS.PubSubPeer(
		onServerConnectionChange
	);

	/**
	 * The server connection state has changed. Here we just log that event, but you
	 * may want to use it to indicate the server is temporarily offline, or similar.
	 */
	function onServerConnectionChange(connected) {
		console.log(connected ? 'Connected to server' : 'Disconnected from server');
	}


	//////////////////////////////////////////////////////////
	// PLAY A BLOCK ON A SPOT
	//////////////////////////////////////////////////////////

	// A selector containering two options, Block1 and Block2
	const blockSelector = document.querySelector('.block-selector');
	// A div showing the currently playing block on "TheSpot"
	const currentPlayingBlockDiv = document.querySelector('#current-playing-block');

	// Wait for the user to select something in the selector.
	blockSelector.addEventListener('change', () => {

		/*	Use the "set" function to set the value of the Spot.TheSpot.block property
			to the composite name of the selected block. This assumes there's a top-
			level Spot in Blocks named TheSpot. Note that the property path syntax is
			the same as used in Tasks and other places in Blocks.

			Here we're passing a string value to the property, since the block
			property expects a string. Pass the proper data type for the type of
			the property here (e.g., string, number or boolean).
		 */
		pubSubPeer.set("Spot.TheSpot.block", blockSelector.value);

	});

	/*	I must define the subscription data handler as a named object,
		so I can  pass the very same object to unsubscribe.
	 */
	const dataHandler = {
		dataReceived: onPlayingBlockChange
	};
	const playingBlockPropName = "Spot.TheSpot.playingBlock";

	/*	Subscribe to changes on the playingBlock property of "TheSpot".
		This is a live subsctiption, that will be notified whenever the
		"playingBlock" changes. Note that the "playingBlock" property is
		different form the "block" property set above. While the two often
		have the same value, they may differ if setting the "block" property
		to a Schedule, or when setting the "priorityBlock" property, which
		then overrides the current "block" property.
	 */
	onPlayingBlockChange(pubSubPeer.subscribe(playingBlockPropName, dataHandler));

	/*	Also show how to unsubscribe from a property. After clicking this button,
		the "Current Playing Block" field will no longer update.
	 */
	document.querySelector('#unsubscribe-button').addEventListener('click', () => {
		pubSubPeer.unsubscribe(playingBlockPropName, dataHandler)
	});

	// Called for any initially available value as well as on value change
	function onPlayingBlockChange(playingBlockName) {
		if (playingBlockName !== undefined) // Initial value may be undefined
			currentPlayingBlockDiv.innerText = playingBlockName;
	}

	//////////////////////////////////////////////////////////
	// CREATE A CHANNEL TO BLOCKS USED TO APPEND TEXT TO A FILE
	//////////////////////////////////////////////////////////

	// The User Script that creates the channel
	const kCommunicatorScriptName = 'Script.user.CustomWebPageCommunicator';

	// The Channel name established by kCommunicatorScriptName
	const kCommunicatorAppendToFileChannellName = 'append-to-file';

	// Build the full composite channel name of the channel
	const fullAppendToFileChannelName = PIXILAB_BLOCKS.PubSubPeer.channelName(
		kCommunicatorScriptName,
		kCommunicatorAppendToFileChannellName
	);

	// Input element specifying the name of the file to append text to
	const fileNameInput = document.querySelector('.file-name');

	// Input element containing the text to be appended
	const textToAppendInput = document.querySelector('.text-to-append');

	// Wait for the user to click the submit button
	document.querySelector('#append-text-button').addEventListener('click', () => {
		console.log(`Sending text to append to "${ fileNameInput.value }"`);

		/*	Create the data object to send, with the file name and the text to append.
			Note that the object created here matches the AppendToFileData interface
			defined in the CustomWebPageCommunicator user script.
		 */
		const appendToFileData = {
			fileName: fileNameInput.value,
			text: textToAppendInput.value,
		};

		// Send the data JSON-encoded through the channel
		pubSubPeer.send(
			fullAppendToFileChannelName,
			JSON.stringify(appendToFileData)
		);
	});


	//////////////////////////////////////////////////////////
	// MAKE A POST (REST) REQUEST TO BLOCKS TO CREATE AN IMAGE
	//////////////////////////////////////////////////////////

	// The div where we eventually will show the final image
	const qrCodeContainerDiv = document.querySelector('.qr-code-container');

	// The input element containing the desired data for the QR code
	const qrContentInput = document.querySelector('.qr-content');

	// Wait for the user to click the submit button.
	const generateQr = document.querySelector('.generate-qr-button');
	generateQr.addEventListener('click', () => {
		// The content defaults to "pixilab" if nothing specified
		const qrCodeContent = qrContentInput.value || 'pixilab';

		// Add a placeholder while waiting for the server call to come back
		qrCodeContainerDiv.innerHTML = ''; // Empty the container div.
		const placeholder = document.createElement('DIV'); // Create a DIV
		placeholder.classList.add('qr-placeholder'); // Set its class
		placeholder.innerText = 'Hold on, creating your QR code...'; // Set its text content
		qrCodeContainerDiv.appendChild(placeholder); // Append it to the container div

		/**
		 * Make a POST request to Blocks. The handler of the POST request is the
		 * generateQRImage function inside of CustomWebPageCommunicator. See more
		 * details in the user script. Note that the data passed along with the
		 * request as JSON matches the QRData interface declared in the
		 * CustomWebPageCommunicator user script.
		 */
		fetch(
			'/rest/script/invoke/CustomWebPageCommunicator/generateQRImage',
			{
				method: 'POST',
				headers: { 'Content-Type': 'application/json' },
				body: JSON.stringify({qrContent: qrCodeContent})
			}
		).then(async response => {
			/*	The image is now ready. The response contains its path.
				If the user script function returns an object, that
				object will be the JSON-encoded response. If the
				user script returns a primitive (as in this case, where
				it returns a string), it will be wrapped in a JSON-
				encoded object with a single "result" field, as seen
				below.
			 */
			const responseObject = await response.json();

			// Rmove the placeholder
			qrCodeContainerDiv.innerHTML = '';

			// Create an IMG element, setting its src to the path of the generated image
			let image = document.createElement('IMG');
			image.src = `${ responseObject.result }?cacheBust=${ new Date().getTime() }`;
			qrCodeContainerDiv.appendChild(image);

			console.log(`Generated QR code image at "${ responseObject.result } has been created."`);
		});
	});
});
