import { Script, ScriptEnv } from "system_lib/Script";
import { SimpleFile } from "system/SimpleFile";
import { resource } from "system_lib/Metadata";
import { WebRenderer } from "system/WebRenderer";

export class CustomWebPageCommunicator extends Script {

	// We use the Google charting API to generate a page with QR code
	private static IMAGE_URL = "https://chart.googleapis.com/chart?cht=qr&chs=250x250&chl=";
	// The folder in which we will place the final QR code image.
	private static CUSTOM_IMAGE_DIR = "/public/custom/";

	/**	User script constructor. Called by Blocks to initialize the script.
	 */
	public constructor(env: ScriptEnv) {
		super(env);

		/**
		 * Establish a websocket channel named "append-to-file", accepting
		 * JSON data representing a AppendToFileData object from the web peer.
		 * Note that such channels aren't restricted to JSON data, but can handle
		 * any text.
		 */
		this.establishChannel(
			"append-to-file",
			data => {
				/*	Parse the data as it is from its JSON string
					whch is expected to render a AppendToFileData object.
				 */
				const appendToFileData: AppendToFileData = JSON.parse(data);
				// Pass the data to doAppendTextToFile, appending it to a file.
				return this.doAppendTextToFile(appendToFileData);
			},
		);

		console.log('CustomWebPageCommunicator started');
	}

	/**
	 * Use Blocks' SimpleFile API to append appendData.text to a file named appendData.fileName
	 * under PIXILAB-Blocks-root/script/files. Creates the file if not already there.
	 * Appends a newline to store each string as a separate line.
	 */
	private doAppendTextToFile(appendData: AppendToFileData): void {
		SimpleFile.append(
			appendData.fileName,
			appendData.text + '\n', // Append newline
		);
	}

	/**
	 * Accept an HTTP POST at /rest/script/invoke/CustomWebPageCommunicator/generateQRImage.
	 * This "screenshots" a web page containing a QR code and then moves the image to a folder
	 * accessible from our custom app.
	 *
	 * See @resource() for details on how the endpoint URL is structured.
	 */
	@resource()
	generateQRImage(data: QRData): Promise<string> {
		const IMAGE_NAME = `qr.png`;	// The name of the final image.

		/*	Since the round-trip and image generateion takes a while, we return a promise.
			Blocks will not return the result of this POST request until the promise
			is resolved.
		 */
		return new Promise((resolve, reject) => {
			/**
			 * Render the small web page containing the QR code to a file.
			 * This is done by calling an open Google API that accepts
			 * the string to be rendered into the QR code as a query
			 * parameter, returning a web page with the requested QR code.
			 * This uses Blocks' built-in WebRenderer API to render the
			 * web page to an image file.
			 */
			WebRenderer.render(
				CustomWebPageCommunicator.IMAGE_URL + data.qrContent,
				IMAGE_NAME,
				250,		// Size of the web page and rendered image, in px
				250,
			).then(() => // Move screenshot to CUSTOM_IMAGE_DIR
				SimpleFile.move(
					"/public/WebRenderer/" + IMAGE_NAME,
					CustomWebPageCommunicator.CUSTOM_IMAGE_DIR + IMAGE_NAME,
					true,
				)
			).then(() => {
				// Return the final file path as the POST request's response
				resolve(CustomWebPageCommunicator.CUSTOM_IMAGE_DIR + IMAGE_NAME);
			}).catch(error => {
				// Something went awry - reject the promise and log the error
				console.error("generateQRImage failed", error)
				reject(error);
			});
		});
	}
}

// Data structure used for the doAppendTextToFile method.
interface AppendToFileData {
	fileName: string;
	text: string;
}

// Data structure used for the generateQRImage method.
interface QRData {
	qrContent: string;
}
